import os
import base64
import requests


class Auth:
    """
    This class handles authentication for the SDK when a config file is not present the user must pass the args below.
    Config file name convention: '.ogc-config'
    Please create .ogc-config in home dir.
    Args:
        host = desired environment
        connectid = the desired account connectid
        username = member manager username
        password = member manager password
    Returns:
        session_object = dictionary containing keys 'base_url', 'connectid', 'headers'
    """

    def __init__(self, base_url=None, connectid=None, username=None, password=None):
        self.base_url = base_url  # host name "https://securewatch.maxar.com/"
        self.connect_id = connectid
        self.username = username
        self.password = password
        self.version = '1.1.0'

        if not self.base_url:
            dir_path = os.path.expanduser('~')
            file = '.ogc-config'
            full_path = os.path.join(dir_path, file)
            if os.path.isfile(full_path):
                self.base_url, self.connect_id, self.username, self.password = self._get_environment(full_path)
            else:
                raise ValueError("Please create .ogc-config in home dir.")
        else:
            acceptable_urls = ["https://securewatch.maxar.com/", "https://securewatch.digitalglobe.com/",
                               "https://access.maxar.com/", "https://services.digitalglobe.com/",
                               "https://evwhs.digitalglobe.com/"]
            if self.base_url not in acceptable_urls:
                raise ValueError("Base_url must match acceptable Maxar url.")

        self.session_object = self._get_session()
        self._check_auth()

    @staticmethod
    def _get_environment(file):
        with open(file) as config_file:
            cred_dict = {}
            for line in config_file.readlines():
                if '=' in line:
                    key, value = line.split('=')
                    key = key.strip()
                    value = value.strip()
                    if '\n' in value:
                        value = value.replace('\n', '')
                    cred_dict.update({key: value})
        tenant = cred_dict['user_tenant']
        connectid = cred_dict['user_connectid']
        try:
            user_name = cred_dict['user_name']
        except:
            user_name = ' '
        try:
            password = cred_dict['user_password']
        except:
            password = ' '
        return tenant, connectid, user_name, password

    def _check_auth(self):
        connectid = self.session_object['connectid']
        host = self.session_object['base_url']
        headers = self.session_object['headers']
        if not connectid:
            raise Exception('Connect ID field is required. Please include a valid Connect ID.')

        if ((not self.username and self.username != ' ') and self.password) or \
                ((not self.password and self.password != ' ') and self.username):
            raise Exception('Username and Password must both be provided.')

        url = "{}catalogservice/wfsaccess?" \
              "REQUEST=GetCapabilities&SERVICE=WFS&VERSION=2.0.0&CONNECTID={}".format(host, connectid)
        response = requests.request("GET", url, headers=headers, data={})
        if response.status_code != 200:
            raise Exception('Unable to connect. Status code equals {}'.format(response.status_code))

    def _get_session(self):
        header = {}
        session = {'base_url': self.base_url,
                   'connectid': self.connect_id,
                   'version': self.version}
        if self.username != ' ' and self.password != ' ':
            header.update({'Authorization': 'Basic {}'.format(self._encode_creds())})
        session.update({'headers': header})
        return session

    def _encode_creds(self):
        auth = "{}:{}".format(self.username, self.password)
        encode = str(base64.b64encode(bytes(auth, 'utf-8')))
        encode = encode[2:-1]
        return str(encode)



import requests
import Maxar_OGC.process as process
from pathlib import Path


class WCS:
    def __init__(self, session):
        self.base_url = session['base_url'] + "deliveryservice/wcsaccess"
        self.headers = session['headers']
        self.connect_id = session['connectid']
        self.response = None
        self.version = session['version']
        self.querystring = self._init_querystring()

    def return_image(self, bbox, identifier, gridoffsets, **kwargs):
        """
        Function finds the imagery matching a bbox or feature id
        Kwargs:
            bbox = String bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)
            filter = CQL filter used to refine data of search.
            gridcrs = String of the Coordinate reference system used. Crs is EPSG:4326.
            gridoffsets = Integer value representing the vertical number of pixels to return
            identifier = String of the feature id to be returned.
            format = String of the format of the response image either jpeg, png or geotiff
            featureprofile = String of the desired stacking profile. Defaults to account Default
        Returns:
            requests response object of desired image
        """

        self.querystring = self._init_querystring()
        keys = list(kwargs.keys())
        process._validate_bbox(bbox)
        self.querystring.update({'boundingbox': bbox})
        self.querystring.update({'identifier': identifier})
        self.querystring.update({'gridoffsets': gridoffsets})

        if 'filter' in keys:
            self.querystring.update({'coverage_cql_filter': kwargs['filter']})
            del (kwargs['filter'])
        for key, value in kwargs.items():
            if key in self.querystring.keys():
                self.querystring[key] = value
            else:
                self.querystring.update({key: value})
        request = requests.get(self.base_url, headers=self.headers, params=self.querystring)
        self.response = request
        return process._response_handler(self.response)

    def _init_querystring(self):
        querystring = {'connectid': self.connect_id,
                       'service': 'WCS',
                       'request': 'GetCoverage',
                       'version': '1.3.0',
                       'gridcrs': 'urn:ogc:def:crs:EPSG::4326',
                       'format': 'image/jpeg',
                       'SDKversion': '{}'.format(self.version)
                       }
        return querystring

    def parse_coverage(self, coverage):
        writelist = []
        filename = Path(coverage)
        filename_replace_ext = filename.with_suffix('.txt')
        with open(coverage, 'rb') as open_file:
            for line in open_file:
                writelist.append(line)
        writelist1 = writelist[:21]
        writelist2 = writelist[21:]
        with open(filename_replace_ext, 'wb') as write_file:
            for item in writelist1:
                write_file.write(item)
        with open(coverage, 'wb') as write_file:
            for item in writelist2:
                write_file.write(item)
        return (str(filename_replace_ext) + " and " + str(coverage) + " have been downloaded")



import requests
import Maxar_OGC.process as process


class WFS:
    def __init__(self, session):
        self.base_url = session['base_url'] + "catalogservice/wfsaccess"
        self.headers = session['headers']
        self.connect_id = session['connectid']
        self.response = None
        self.version = session['version']
        self.querystring = self._init_querystring()

    def search(self, typename='FinishedFeature', **kwargs):
        """
        Function searches using the wfs method.
        Args:
            typename = String of the typename. Defaults to 'FinishedFeature'. Example input 'MaxarCatalogMosaicProducts'
        Kwargs:
            bbox = String bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)
            filter = CQL filter used to refine data of search.
            outputformat = String of the format of the response object. Defaults to json.
            featureprofile = String of the desired stacking profile. Defaults to account Default
            typename = String of the typename. Defaults to FinishedFeature. Example input MaxarCatalogMosaicProducts
        Returns:
            Response object of the search
        """

        self.querystring = self._init_querystring()
        process._check_typeName(typename)
        self.querystring.update({'typename': 'DigitalGlobe:{}'.format(typename)})
        keys = list(kwargs.keys())
        if 'filter' in keys and kwargs['filter']:
            if 'bbox' in keys and kwargs['bbox']:
                process._validate_bbox(kwargs['bbox'])
                bbox_list = [i for i in kwargs['bbox'].split(',')]
                if len(bbox_list) != 4:
                    raise Exception('Only EPSG:4326 valid for filter')
                coords = ','.join(bbox_list[:4])
                process._validate_bbox(coords)
                self._combine_bbox_and_filter(kwargs['filter'], kwargs['bbox'], typename)
                del(kwargs['filter'])
                del(kwargs['bbox'])
            else:
                self.querystring.update({'cql_filter': kwargs['filter']})
                del (kwargs['filter'])
        elif 'bbox' in keys:
            process._validate_bbox(kwargs['bbox'])
            self.querystring.update({'bbox': kwargs['bbox']})
            del(kwargs['bbox'])
        else:
            raise Exception('Search function must have a BBOX or a Filter.')
        for key, value in kwargs.items():
            self.querystring[key] = value

        query_string = process._remove_cache(self.querystring)
        request = requests.get(self.base_url, headers=self.headers, params=query_string)
        self.response = request
        return process._response_handler(self.response)

    def _combine_bbox_and_filter(self, filter, bbox, typename):

        if 'MaxarCatalogMosaic' in typename:
            bbox_geometry = 'BBOX(shape,{})'.format(bbox)
            combined_filter = bbox_geometry + 'AND' + '(' + filter + ')'
        else:
            bbox_geometry = 'BBOX(geometry,{})'.format(bbox)
            combined_filter = bbox_geometry + 'AND' + '(' + filter + ')'
        self.querystring.update({'cql_filter': filter})
        self.querystring.update({'cql_filter': combined_filter})

    def _init_querystring(self):
        querystring = {'connectid': self.connect_id,
                       'service': 'WFS',
                       'request': 'GetFeature',
                       'typename': 'DigitalGlobe:FinishedFeature',
                       'version': '1.1.0',
                       'srsname': 'EPSG:4326',
                       'height': '3000',
                       'width': '3000',
                       'outputformat': 'json',
                       'SDKversion' : '{}'.format(self.version)
                       }
        return querystring




import os

from Maxar_OGC.auth import Auth
from Maxar_OGC.wms import WMS
from Maxar_OGC.wfs import WFS
from Maxar_OGC.wmts import WMTS
from Maxar_OGC.wcs import WCS
import Maxar_OGC.process as process
import requests
import warnings
import queue
import concurrent.futures
from concurrent.futures import as_completed
import sys
from PIL import Image


warnings.filterwarnings("ignore")


class Interface:
    """
    The primary interface for interacting with the WMS and WFS OGC classes.
    Args:
        base_url (string) = The url that you are using ex. 'https://securewatch.digitalglobe.com/'
        connect_id (string) = The connectId tied to your account
        username (string) = The username if your connectId requires Auth
        password (string) = The password associated with your username
    """

    def __init__(self, *args):
        if len(args) > 0:
            try:
                base_url = args[0]
                connect_id = args[1]
            except:
                raise Exception("Must pass connectId with base_url")
            if len(args) == 4:
                username = args[2]
                password = args[3]
                self._session = Auth(base_url, connect_id, username, password).session_object
            else:
                self._session = Auth(base_url, connect_id).session_object
        else:
            self._session = Auth().session_object

        self.wms = WMS(self._session)
        self.wfs = WFS(self._session)
        self.wmts = WMTS(self._session)
        self.wcs = WCS(self._session)

    def search(self, bbox=None, filter=None, shapefile=False, **kwargs):
        """
        Function searches using the wfs method.
        Args:
            bbox (string) = Bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)
            filter (string) = CQL filter used to refine data of search.
            shapefile (bool) = Binary of whether or not to return as shapefile format
        Kwargs:
            featureprofile (string) = The desired stacking profile. Defaults to account Default
            typename (string) = The typename of the desired feature type. Defaults to FinishedFeature. Example input
            MaxarCatalogMosaicProducts
        Returns:
            Response is either a list of features or a shapefile of all features and associated metdata.
        """

        if shapefile:
            result = self.wfs.search(bbox=bbox, filter=filter, outputformat='shape-zip', **kwargs)
        else:
            result = self.wfs.search(bbox=bbox, filter=filter, **kwargs)
        if bbox:
            if shapefile:
                return process.download_file(result, format_response='zip')
            else:
                result = process.aoi_coverage(bbox, result)
                return result['features']

        else:
            if shapefile:
                return process.download_file(result, format_response='zip')
            else:
                return result.json()['features']

    def download_image(self, bbox=None, height=None, width=None, img_format=None, identifier=None,
                       gridoffsets=None, zoom_level=None, download=True, outputpath=None, display=True,
                       **kwargs):
        """
        Function downloads the image using the wms method.
        Args:
            bbox (string) = Bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)
            height (int) = The vertical number of pixels to return
            width (int) = The horizontal number of pixels to return
            img_format (string) = The format of the response image either jpeg, png or geotiff
            identifier (string) = The feature id
            gridoffsets (string) = The pixel size to be returned in X and Y dimensions
            zoom_level (int) = The zoom level. Used for WMTS
            download (bool) = User option to download band manipulation file locally.
            outputpath (string) = Output path must include output format. Downloaded path default is user home path.
        Kwargs:
            legacyid (string) = The duc id to download the browse image
        Returns:
            requests response object or downloaded file path
        """
        acceptable_format = ['jpeg', 'png', 'geotiff']
        if img_format in acceptable_format:
            img_formatted = 'image/' + img_format
        else:
            raise Exception('Format not recognized, please use acceptable format for downloading image.')
        if 'legacyid' in kwargs.keys():
            legacy_id = kwargs['legacyid']
            url = "https://api.discover.digitalglobe.com/show?id={}".format(legacy_id)
            result = requests.request("GET", url, headers={}, data={})
        elif zoom_level:
            if not bbox:
                raise Exception('zoom_level must have a bbox')
            else:
                process._validate_bbox(bbox)
                wmts_list = self.wmts.wmts_bbox_get_tile_list(zoom_level, bbox)
                return wmts_list
        elif identifier:
            if not gridoffsets or not bbox:
                raise Exception('Identifiers must have gridoffset and bbox')
            else:
                result = self.wcs.return_image(bbox, identifier, gridoffsets)
                if outputpath:
                    file_name = process.download_file(result, download_path=outputpath)
                else:
                    file_name = process.download_file(result, format_response=img_format)
                return self.wcs.parse_coverage(file_name)
        else:
            if not bbox or not img_format or not width or not height:
                raise Exception('height/width must have a bbox and an img_format')
            else:
                process._validate_bbox(bbox)
                if width < 0 or width > 8000:
                    raise Exception("Invalid value for width parameter (max 8000)")
                if height < 0 or height > 8000:
                    raise Exception("Invalid value for height parameter (max 8000)")
                result = self.wms.return_image(bbox=bbox, format=img_formatted, height=height, width=width, **kwargs)
        if display:
            process._display_image(result)
        if download:
            if outputpath:
                file_name = process.download_file(result, download_path=outputpath)
            else:
                file_name = process.download_file(result, format_response=img_format)
            return f"Downloaded file {file_name}"
        else:
            return result

    def download_browse_image(self, input_id, img_format='jpeg', outputpath=None, display=False):
        """
        Function downloads the browse image for the desired legacy id
        Args:
            input_id (string) = The desired input id (Can be feature id or catalog id)
            img_format (string) = The format of the response image either jpeg, png or geotiff
            outputpath (string) = Output path must include output format. Downloaded path default is user home path.
            display (bool) = Display image in IDE (Jupyter Notebooks only)
        Returns:
            Downloaded image location of desired legacy id in desired format
        """

        legacyid = self._convert_feature_to_legacy(input_id)
        process._check_image_format(img_format)
        url = "https://api.discover.digitalglobe.com/show?id={}".format(legacyid)
        result = requests.request("GET", url, headers={}, data={})
        if display:
            process._display_image(result)
        if outputpath:
            file_name = process.download_file(result, download_path=outputpath)
        else:
            file_name = process.download_file(result, format_response=img_format)
        return f"Downloaded file {file_name}"

    def get_tile_list_with_zoom(self, bbox, zoom_level):
        """
        Function acquires a list of tile calls dependent on the desired bbox and zoom level
        Args:
            bbox (string) = Bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)
            zoom_level (int) = The zoom level
        Returns:
            List of individual tile calls for desired bbox and zoom level
        """

        process._validate_bbox(bbox)
        wmts_list = self.wmts.wmts_bbox_get_tile_list(zoom_level, bbox)
        return wmts_list

    def download_tiles(self, bbox, zoom_level, img_format='jpeg', outputpath=None, display=False):
        """
        Function downloads all tiles within a bbox dependent on zoom level
        Args:
            bbox (string) = Bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)
            zoom_level (int) = The zoom level
            img_format (string) = The format of the response image either jpeg, png or geotiff
            outputpath (string) = Output path must include output format. Downloaded path default is user home path.
            display (bool) = Display image in IDE (Jupyter Notebooks only)
        Returns:
            Message displaying success and location of downloaded tiles
        """

        wmts = self.get_tile_list_with_zoom(bbox, zoom_level)[1]
        if outputpath:
            extension = outputpath.split(".")[-1]
            base_file = outputpath.replace("." + extension, "")
        else:
            extension = img_format
            base_file = os.getcwd() + "\\Download"
        for tile in wmts:
            response = self.wmts.wmts_get_tile(tile[0], tile[1], tile[2])
            if display:
                process._display_image(response)
            filename = "{}_{}_{}_{}.{}".format(base_file, tile[0], tile[1], tile[2], extension)
            with open(filename, 'wb+') as f:
                f.write(response.content)
        return "Download complete, files are located in {}".format(base_file)

    def download_image_with_feature_id(self, bbox, identifier, gridoffsets, img_format='jpeg', display=True,
                                       outputpath=None):
        """
        Function downloads the image and metadata of desired feature id
        Args:
            bbox (string) = Bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)
            identifier (string) = Desired feature id
            gridoffsets (string) = The pixel size to be returned in X and Y dimensions
            img_format (string) = The format of the response image either jpeg, png or geotiff
            outputpath (string) = Output path must include output format. Downloaded path default is user home path.
        Returns:
            Downloaded image location of desired feature id in desired format and associated metadata
        """

        process._check_image_format(img_format)
        result = self.wcs.return_image(bbox, identifier, gridoffsets)

        if display:
            process._display_image(result)

        if outputpath:
            file_name = process.download_file(result, download_path=outputpath)
        else:
            file_name = process.download_file(result, format_response=img_format)
        return self.wcs.parse_coverage(file_name)

    def download_image_by_pixel_count(self, bbox, height, width, img_format='jpeg', outputpath=None, display=False,
                                      **kwargs):
        """
        Function downloads the image of desired bbox dependent on pixel height and width
        Args:
            bbox (string) = Bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)
            height (int) = The vertical number of pixels to return
            width (int) = The horizontal number of pixels to return
            img_format (string) = The format of the response image either jpeg, png or geotiff
            outputpath (string) = Output path must include output format. Downloaded path default is user home path.
            display (bool) = Display image in IDE (Jupyter Notebooks only)
        Kwargs:
            filter (string) = CQL filter used to refine data of search.
            featureprofile (string) = The desired stacking profile. Defaults to account Default
            bands (list[string]) = The desired band combination of 1-4 items. Requires SWIR 8 Band or MS1_MS2
        Returns:
            Downloaded image location of desired bbox dependent on pixel height and width
        """

        img_formatted = process._check_image_format(img_format)
        process._validate_bbox(bbox)
        if width <= 0 or width > 8000:
            raise Exception("Invalid value for width parameter (max 8000)")
        if height <= 0 or height > 8000:
            raise Exception("Invalid value for height parameter (max 8000)")
        result = self.wms.return_image(bbox=bbox, format=img_formatted, height=height, width=width, **kwargs)
        if display:
            process._display_image(result)

        if outputpath:
            file_name = process.download_file(result, download_path=outputpath)
        else:
            file_name = process.download_file(result, format_response=img_format)
        return "Downloaded file {}".format(file_name)

    def band_manipulation(self, bbox, featureid, band_combination, height=256, width=256, img_format='jpeg',
                          display=True,
                          outputpath=None):
        """
        Function changes the bands of the feature id passed in.
        Args:
            bbox (string) = Bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)
            featureid (string) = The id of the image
            band_combination (list[string]) = The desired band combination of 1-4 items.
            height (int) = The vertical number of pixels to return
            width (int) = The horizontal number of pixels to return
            image_format (string) = The file type that you want downloaded.
            outputpath (string) = Output path must include output format. Downloaded path default is user home path.
        Returns:
            download location for file
        """

        band_string = self._band_check(featureid, band_combination)
        feature_id_filter = "featureId='{}'".format(featureid)
        message = self.download_image_by_pixel_count(bbox, height, width, img_format, outputpath=outputpath,
                                                     display=display,
                                                     filter=feature_id_filter, bands=band_string)
        return message

    def get_full_res_image(self, featureid, thread_number=100, bbox=None, mosaic=False, **kwargs):
        """
        Function takes in a feature id and breaks the image up into 1024x1024 tiles, then places a number of calls
        based on multithreading percentages to return a full image strip in multiple tiles
        Args:
            featureid (string) = Feature id of the image
            thread_number (int) = Number of threads given to multithread functionality
            bbox (string) = Bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)
            mosaic (bool) = Flag if image files are mosaiced
        kwargs:
            outputdirectory (string) = Desired output location for tiles
            image_format (string) = Desired image format (png or jpeg)
        Returns:
            None
        """

        if bbox:
            process._validate_bbox(bbox)
        wfs_request = self.search(filter="featureId='{}'".format(featureid))
        image_bbox = wfs_request[0]['geometry']['coordinates'][0]
        x_coords = [x[0] for x in image_bbox]
        y_coords = [y[1] for y in image_bbox]

        miny = min(y_coords)
        maxy = max(y_coords) + 0.0042176
        minx = min(x_coords)
        maxx = max(x_coords) + 0.0054932

        if bbox:
            bbox_order = bbox.split(',')
            miny = max(miny, float(bbox_order[0]))
            maxy = min(maxy, float(bbox_order[2]))
            minx = max(minx, float(bbox_order[1]))
            maxx = min(maxx, float(bbox_order[3]))

        y_list = []
        x_list = []
        while miny < maxy:
            y_list.append(miny)
            miny += 0.0042176
        while minx < maxx:
            x_list.append(minx)
            minx += 0.0054932
        tiles = {}

        if len(y_list) == 1:
            if len(x_list) == 1:
                tiles['c{}_r{}'.format(0, 0)] = '{}, {}, {}, {}'.format(y_list[0], x_list[0],
                                                                        y_list[0] + 0.0042176, x_list[0] + 0.0054932)
            else:
                for x in range(len(x_list) - 1):
                    tiles['c{}_r{}'.format(x, 0)] = '{}, {}, {}, {}'.format(y_list[0], x_list[x], y_list[0] + 0.0042176,
                                                                            x_list[x + 1])
        elif len(x_list) == 1:
            for y in reversed(range(len(y_list) - 1)):
                tiles['c{}_r{}'.format(0, len(y_list) - y - 2)] = '{}, {}, {}, {}'.format(y_list[y], x_list[0],
                                                                                          y_list[y + 1],
                                                                                          x_list[0] + 0.0054932)
        else:
            for y in reversed(range(len(y_list) - 1)):
                for x in range(len(x_list) - 1):
                    tiles['c{}_r{}'.format(x, len(y_list) - y - 2)] = '{}, {}, {}, {}'.format(y_list[y], x_list[x],
                                                                                              y_list[y + 1],
                                                                                              x_list[x + 1])

        print("Started full image download process...")

        url = self.wms.base_url
        headers = self.wms.headers
        querstring = self.wms.querystring
        querstring['width'] = 1024
        querstring['height'] = 1024
        querstring['coverage_cql_filter'] = "featureId='{}'".format(featureid)
        if 'image_format' in kwargs.keys():
            format = kwargs['image_format']
            querstring['format'] = 'image/{}'.format(format)
        else:
            format = querstring['format'][6:]
        response_times = {}

        def response_thread(coord_list):
            """
            Function multithreads requests to speed up image return process
            Args:
                coord_list (list) = Coordinates for individual tiles
            Returns:
                List of cell locations and corresponding response objects
            """
            sub_bbox, sub_grid_cell_location = coord_list.split("|")
            sub_query = querstring.copy()
            sub_query['bbox'] = sub_bbox
            sub_response = requests.request("GET", url, params=sub_query, headers=headers)
            return [sub_grid_cell_location, sub_response]


        multithreading_array = ["{}|{}".format(k, j) for j, k in list(tiles.items())]
        if 'outputdirectory' not in kwargs.keys():
            outputdirectory = os.path.expanduser('~')
        else:
            outputdirectory = kwargs['outputdirectory']
        with open(os.path.join(outputdirectory, 'Grid_cell_coordinates.txt'), 'w') as grid_coords:
            grid_coords.write('grid_cell_name | grid_cell_bbox\n')
            for line in multithreading_array:
                value, key = line.split('|')
                grid_coords.write('{} | {}\n'.format(key, value))

        chunk_size = thread_number * 5
        chunk_count = int(len(multithreading_array) / chunk_size)
        for i in range(chunk_count + 1):
            if i == chunk_count:
                sub_array = multithreading_array[i * chunk_size:]
            else:
                sub_array = multithreading_array[i * chunk_size:(i + 1) * chunk_size]
            with process.BoundedThreadPoolExecutor(max_workers=thread_number) as executor:
                futures = [executor.submit(response_thread, coords) for coords in sub_array]
                for future in as_completed(futures):
                    coord, response = future.result()
                    sub_output = os.path.join(outputdirectory, coord + ".{}".format(format))
                    process.download_file(response, download_path=sub_output)
            sys.stdout.write('Finished section {} out of {}'.format(i+1, chunk_count+1))
            sys.stdout.write('\r')
        sys.stdout.write('\r')
        print('\n')

        if mosaic:
            return "Finished full image download process, output directory is: {}. Beginning mosaic process".\
                format(os.path.split(outputdirectory)[0])
            create_mosaic(base_dir=kwargs['outputdirectory'], img_size=1024, img_format=image_format, **kwargs)
        else:
            return "Finished full image download process, output directory is: {}".\
                format(os.path.split(outputdirectory)[0])


    def create_mosaic(self, base_dir, img_size=1024, img_format='png', **kwargs):
        '''
        Function creates a mosaic of downloaded image tiles from full_res_dowload function
        Args:
            base_dir (string) = Root directory containing image files to be mosaiced
            img_size (int) = Size of individual image files, defaults to 1024
            img_format (string) = Image format of files
        Kwargs:
            outputdirectory (string) = Directory destination of finished mosaic file
        Returns:
            None
        '''
        Image.MAX_IMAGE_PIXELS = None
        coord_list = []
        for k in [i for i in os.listdir(base_dir) if ".txt" not in i and os.path.isfile(os.path.join(base_dir, i))]:
            filename = k
            coords = k.replace('c', '').replace('_r', ',').replace('.{}'.format(img_format), '').split(',')
            coord_list.append([filename, int(coords[0]), int(coords[1])])

        max_row = max([i[2] for i in coord_list]) + 1
        max_col = max([i[1] for i in coord_list]) + 1
        maximum = max(max_col, max_row)
        size = img_size * maximum
        mosaic = Image.new('RGB', (max_col * img_size, max_row * img_size), (size, size, size))

        count = 0
        for i in coord_list:
            column = img_size * i[1]
            row = img_size * i[2]
            mosaic.paste(Image.open(os.path.join(base_dir, i[0])), (column, row))
            count += 1
            if count % 100 == 0:
                sys.stdout.write("Processing {} of {} total".format(count, len(coord_list)))
                sys.stdout.write("\r")

        if 'outputdirectory' in kwargs.keys():
            mosaic.save(r"{}\merged_image.{}".format(kwargs['outputdirectory'], img_format))
            sys.stdout.write("Finished image mosaic process, output directory is: {}".format(kwargs['outputdirectory']))
        else:
            mosaic.save(r"{}\merged_image.{}".format(base_dir, img_format))
            sys.stdout.write("Finished image mosaic process, output directory is: {}".format(base_dir))

    def _band_check(self, featureid, band_combination):
        """
        Function checks bands given against a list of valid bands
        Args:
            featureid (string) = The id of the image
            band_combination (list[string]) = The desired band combination of 1-4 items.
        Returns:
            String of band combination
        """
        band_check = self.search(filter="featureId='{}'".format(featureid))
        band_check_list = ['MS1_MS2', 'SWIR 8-Band']
        if band_check[0]['properties']['productType'] not in band_check_list:
            raise Exception('Product Type for the image must be either SWIR 8-band or MS1_MS2.')
        band_options = ['R', 'G', 'B', 'C', 'Y', 'RE', 'N', 'N2', 'S1', 'S2', 'S3', 'S4', 'S5', 'S6', 'S7']
        if len(band_combination) <= 0 or len(band_combination) > 4:
            raise Exception('The number of bands must be greater than 0 and less than or equal to 4.')

        band_string = ','.join([i if i in band_options else 'z' for i in band_combination])

        if 'z' in band_string:
            raise Exception(band_combination + ' is not a valid option.')
        return band_string

    def _convert_feature_to_legacy(self, input_id):
        """
        Function takes in a feature id or legacy id and finds the browse image associated with it.
        Args:
            input_id (string) = The id that you are searching for
        Returns:
            Legacy id of desired feature
        """

        catalog_identifiers = ['101', '102', '103', '104', '105', '106']

        # if the id passed in is a cat id or WVO4 Inv id. Return the browse for that id from discover api
        if input_id[0:3] in catalog_identifiers or '-inv' == input_id[-4:]:
            legacy_id = input_id
        # If the id passed in is a feature id. Use our wfs method to return a json and parse out the legacy id from
        # the metadata
        else:
            json_return = self.search(filter="featureId='{}'".format(input_id))
            legacy_id = json_return[0]['properties']['legacyId']
        return legacy_id

    @staticmethod
    def calculate_sqkm(bbox):
        """
        Function calculates the area in square kilometers of the desired bounding box
        Args:
            bbox (string) = Bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)
        Returns:
            Float of bounding box area in square kilometers
        """
        
        area = process.area_sqkm(bbox)
        return area



import requests
import xml.etree.ElementTree as ET
import Maxar_OGC.process as process


class WMTS:
    def __init__(self, session):
        self.base_url = session['base_url'] + "earthservice/wmtsaccess"
        self.headers = session['headers']
        self.connect_id = session['connectid']
        self.response = None
        self.version = session['version']
        self.querystring = self._init_querystring()

    def wmts_convert(self, zoom_level, laty, longx):
        """
        Function converts a lat long position to the tile column and row needed to return WMTS imagery over the area
        Args:
            zoom_level: Integer value of the desired zoom level
            laty: Integer value of the latitude
            longx: Integer value of the desired longitude
        Returns:
            String values of the Tile Row and the Tile Column
        """

        querystring = self.querystring
        querystring['request'] = 'GetCapabilities'
        matrixwidth = ''
        matrixheight = ''
        projection = 'EPSG:4326'
        response = requests.request("GET", self.base_url, headers=self.headers, params=querystring)
        response = process._response_handler(response)
        root = ET.fromstring(response.content)
        for TileMatrixSet in root.iter(r'{http://www.opengis.net/wmts/1.0}TileMatrixSet'):
            if len(TileMatrixSet) > 0:
                for i in range(2, 23):
                    if TileMatrixSet[i][0].text == projection + ':' + str(zoom_level):
                        matrixwidth = TileMatrixSet[i][5].text
                        matrixheight = TileMatrixSet[i][6].text
        if not matrixwidth or not matrixheight:
            raise Exception('Unable to determine Matrix dimensions from input coordinates')

        return str(round((float(longx) + 180) * (int(matrixwidth)/360))), str(round((90 - float(laty)) * (int(matrixheight)/180)))

    def wmts_get_tile(self, tilerow, tilecol, zoom_level, **kwargs):
        """
        Function executes the wmts call and returns a response object of the desired tile
        Args:
            tilerow: String value of the tile row.
            tilecol: String value of the tile column
            zoom_level: Integer value of the desired zoom level
        Returns:
            WMTS tiles for the input data
        """

        querystring = self.querystring
        querystring['TileMatrix'] = querystring['TileMatrixSet'] + ':' + str(zoom_level)
        querystring['tilerow'] = tilerow
        querystring['tilecol'] = tilecol
        querystring['request'] = 'GetTile'
        response = requests.request("GET", self.base_url, headers=self.headers, params=self.querystring)
        process_response = process._response_handler(response)
        return process_response

    def wmts_bbox_get_tile_list(self, zoom_level, bbox, **kwargs):
        """
        Function takes in a bbox and zoom level to return a list of WMTS calls that can be used to aquire all the wmts
        tiles. Projection is EPSG:4326
        Args:
            zoom_level: Integer value of the desired zoom level
            bbox: String bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)
        Returns:
            List of WMTS calls.
        """

        process._validate_bbox(bbox)
        bbox_list = [i for i in bbox.split(',')]
        if len(bbox_list) == 4:
            miny = float(bbox_list[0])
            minx = float(bbox_list[1])
            maxy = float(bbox_list[2])
            maxx = float(bbox_list[3])
        else:
            raise Exception('Must provide four coordinates in EPSG:4326')

        min_tilerow, min_tilecol = self.wmts_convert(zoom_level, miny, minx)
        max_tilerow, max_tilecol = self.wmts_convert(zoom_level, maxy, maxx)
        if max_tilerow < min_tilerow:
            swap = max_tilerow
            max_tilerow = min_tilerow
            min_tilerow = swap
        if max_tilecol < min_tilecol:
            swap = max_tilecol
            max_tilecol = min_tilecol
            min_tilecol = swap
        tiles = []
        row_col = []

        for i in range(int(min_tilecol), int(max_tilecol) + 1):
            for j in range(int(min_tilerow), int(max_tilerow) + 1):
                querystring = self.querystring
                querystring['request'] = 'GetTile'
                querystring['TileMatrix'] = querystring['TileMatrixSet'] + ':' + str(zoom_level)
                querystring['TileRow'] = i
                querystring['TileCol'] = j
                tiles.append(self.base_url + '?' + "&".join("{}={}".format(key, value) for key,value in querystring.items()))
                row_col.append((querystring['TileRow'], querystring['TileCol'], zoom_level))
        combined = [tiles, row_col]
        return combined

    def _init_querystring(self):
        querystring = {'connectid': self.connect_id,
                       'service': 'WMTS',
                       'request': 'GetTile',
                       'version': '1.0.0',
                       'TileMatrixSet': 'EPSG:4326',
                       'Layer': 'DigitalGlobe:ImageryTileService',
                       'Format': 'image/jpeg',
                       'SDKversion': '{}'.format(self.version)
                       }
        return querystring




import os
import pyproj
import shapely.ops as ops
from shapely.geometry.polygon import Polygon
import shapely.geometry
import shapely.wkt
from functools import partial
import random
import string
import queue
from concurrent.futures import ThreadPoolExecutor



def _response_handler(response):
    """
    Function takes in the server response code and responds accordingly.
    Returns:
        requests response object of server status
    """
    if response.status_code != 200:
        raise Exception("Non-200 response received for {}.".format(response.url))
    elif 'Exception' in response.text:
        raise Exception(response.url, response.text)
    else:
        return response


def area_sqkm(bbox):
    """
    Function takes in the bbox and calculates the area in SQKM.
    Args:
        bbox =  String of Coordinates separated by comma
            ex: "39.84387,-105.05608,39.95133,-104.94827"
    Returns:
        float value of area in SQKM
    """

    _validate_bbox(bbox)
    bboxlst = bbox.split(',')
    ymin = float(bboxlst[0])
    ymax = float(bboxlst[2])
    xmin = float(bboxlst[1])
    xmax = float(bboxlst[3])

    geom = Polygon([(xmin, ymin), (xmin, ymax), (xmax, ymax), (xmax, ymin), (xmin, ymin)])

    geom_area = ops.transform(
    partial(
        pyproj.transform,
        pyproj.Proj(init='EPSG:4326'),
        pyproj.Proj(
            proj='aea',
            lat_1=geom.bounds[1],
            lat_2=geom.bounds[3]
        )
    ),
    geom)

    # Print the area in sqkm^2

    geomareasqkm = geom_area.area/(10**6)
    return geomareasqkm


def _display_image(image):
    """
    Function takes in the response object and displays it.
    Args:
        image = response object from wms method
    """

    try:
        import IPython.display as disp
        from IPython.display import Image, display
    except:
        raise Exception('Must have IPython installed to display.')
    display(disp.Image(image.content))


def _validate_bbox(bbox):
    """
    Function takes in the bbox and validates that it is proper format
    Args:
        bbox =  String of Coordinates separated by comma
        example = "-105.05608, 39.84387, -104.94827, 39.95133"
    """

    bbox_list = [i for i in bbox.split(',')]
    if len(bbox_list) == 4:
        try:
            bbox_data = {'min_y': float(bbox_list[0]), 'min_x': float(bbox_list[1]), 'max_y': float(bbox_list[2]),
                         'max_x': float(bbox_list[3])}
        except:
            raise Exception('Bbox coordiantes must be numeric.')
        if bbox_data['min_y'] >= bbox_data['max_y']:
            raise Exception("Improper order of bbox: min_y is greater than max_y.")
        if bbox_data['min_x'] >= bbox_data['max_x']:
            raise Exception("Improper order of bbox: min_x is greater than max_x.")

        if [bbox_data[i] for i in bbox_data.keys() if '_y' in i if bbox_data[i] > 90 or bbox_data[i] < -90]:
            if [bbox_data[i] for i in bbox_data.keys() if '_x' in i if bbox_data[i] > 180 or bbox_data[i] < -180]:
                raise Exception(
                    "Improper bbox parameter: {} coordinate outside of range -90:90 and {} outside of range -180:180".format(
                        (bbox_list[0], bbox_list[2]), (bbox_list[1], bbox_list[3])))
            else:
                raise Exception("Improper bbox parameter: {} coordinate outside of range -90:90.".format(
                    (bbox_list[0], bbox_list[2])))
        elif [bbox_data[i] for i in bbox_data.keys() if '_x' in i if bbox_data[i] > 180 or bbox_data[i] < -180]:
            raise Exception("Improper bbox parameter: {} coordinate outside of range -180:180.".format(
                (bbox_list[1], bbox_list[3])))

    else:
        raise Exception("Projection must be EPSG:4326 (exactly 4 coordinates)")


def download_file(response, format_response=None, download_path=None):
    """
    Function downloads response to local machine
    Args:
        response: Dictionary of response object
        format_response: String of file type
        download_path: String of desired download path location on local machine
    Returns:
        String of download path location on local machine
    """

    if download_path:
        filename = download_path
    else:
        filename = 'Download.' + format_response
        filename = os.path.join(os.getcwd(), filename)
    if os.path.isfile(filename):
        while os.path.isfile(filename):
            filename = os.path.join(os.path.split(filename)[0], os.path.split(filename)[1].split('.')[0] + '_dup.' +
                                    os.path.split(filename)[1].split('.')[1])
    with open(filename, 'wb') as output_file:
        output_file.write(response.content)

    return filename


def _remove_cache(querystring):
    """
    Function assigns random characters to bypass caching
    Args:
        querystring: Dictionary of the desired query
    Returns:
        Dictionary of updated query string with random characters
    """

    pool_list = string.digits + string.ascii_letters
    random_characters1 = ''.join(i for i in random.choices(pool_list, k=25))
    random_characters2 = ''.join(i for i in random.choices(pool_list, k=25))
    querystring.update({random_characters1:random_characters2})
    return querystring

def aoi_coverage(bbox, response):
    """
    Function adds the percentage of the desired feature that is covered by the AOI
    Args:
        bbox: String of Coordinates separated by comma
        response: Response object from a WFS request call
    Returns:
        Updated dictionary of the json object
    """

    coverage = response.json()
    box_order = bbox.split(',')
    aoi_polygon = shapely.geometry.box(
        *(float(box_order[1]), float(box_order[0]), float(box_order[3]), float(box_order[2])), ccw=True)
    for feature in coverage['features']:
        if feature['geometry']['type'] == 'Polygon':
            feature_wkt = shapely.wkt.loads("POLYGON (({}))".format(
                ", ".join([" ".join([str(k) for k in i]) for i in feature['geometry']['coordinates'][0]])))
            feature['coverage'] = aoi_polygon.intersection(feature_wkt).area / feature_wkt.area
        else:
            feature_wkt = shapely.wkt.loads("MULTIPOLYGON ({})".format(", ".join(
                ["(({}))".format(", ".join([" ".join([str(k) for k in i]) for i in l[0]])) for l in
                 feature['geometry']['coordinates']])))
            feature['coverage'] = aoi_polygon.intersection(feature_wkt).area / feature_wkt.area

    return coverage

def _check_image_format(img_format):
    """
    Function checks image format given for each call
    Args:
        img_format: String of desired image format
    Returns:
        String of image format with 'image/' preceding to comply with Maxar API calls
    """

    acceptable_format = ['jpeg', 'png', 'geotiff']
    if img_format in acceptable_format:
        return 'image/' + img_format
    else:
        raise Exception('Format not recognized, please use acceptable format for downloading image.')

def _check_typeName(typename):
    acceptable_types = ['FinishedFeature', 'TileMatrixFeature', 'ImageInMosaicFeature', 'MaxarCatalogMosaicProducts',
                        'MaxarCatalogMosaicSeamlines', 'MaxarCatalogMosaicTiles']
    if typename not in acceptable_types:
        raise Exception('{} is not an acceptable TypeName. Please use one of the following {}'.format(typename, acceptable_types))

class BoundedThreadPoolExecutor(ThreadPoolExecutor):

    def __init__(self, *args, **kwargs):
        super(BoundedThreadPoolExecutor, self).__init__(*args, **kwargs)
        self._work_queue = queue.Queue()


import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent.resolve()

README = (HERE / "README.md").read_text()

setup(
  name = 'Maxar_OGC',
  version = '1.1.0',
  license='MIT',
  description = 'SDK for interacting with Maxar imagery platforms',
  long_description=README,
  long_description_content_type="text/markdown",
  author = 'GCS Marianas Team',
  author_email = 'DL-GCS-Marianas@digitalglobe.com',
  project_urls = {
        'Documentation': 'https://cloudservices.readthedocs.io/en/latest/index.html',
        'Source': 'https://github.com/DigitalGlobe/CloudServices'
        },
  keywords = ['OGC', 'WMS', 'WFS', 'WMTS', 'WCS', 'MAXAR', 'IMAGERY', 'GIS'],
  python_requires= '>=3.6',
  install_requires=[
          'pyproj',
          'shapely',
          'requests',
          'ipython',
	  'pillow',
          'click'
      ],
  classifiers=[
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
  ],
  entry_points='''
    [console_scripts]
    search=Maxar_OGC.cli_commands:search
    config=Maxar_OGC.cli_commands:config_file
    password=Maxar_OGC.cli_commands:reset_password
    download=Maxar_OGC.cli_commands:download
    bands=Maxar_OGC.cli_commands:band_manipulation
    area=Maxar_OGC.cli_commands:calculate_bbox_sqkm
    '''
)



import click
import sys
import os
from Maxar_OGC.interface import Interface


def check_for_config():
    home_dir = os.path.expanduser("~")
    if os.path.exists(os.path.join(home_dir, '.ogc-config')):
        cli_interface = Interface()
    else:
        raise Exception("OGC config file not found. Please ensure that your config file is located in your home "
                        "directory")
    return cli_interface


@click.command()
@click.option('--tenant', '-t', help='Desired tenant', type=str, prompt='Please enter tenant')
@click.option('--connectid', '-c', help='Your connect Id', type=str, prompt='Please enter connect Id')
@click.option('--username', '-u', help='Your username', type=str, prompt='Please enter your username if required',
              default=' ')
@click.option('--password', '-p', help='Your password', type=str, prompt='Please enter your password if required',
              default=' ')
def config_file(tenant, connectid, username=None, password=None):
    """
    Function creates a configuration file for authentication use for Maxar tenants
    """

    if tenant.lower() == "securewatch":
        domain = "https://securewatch.maxar.com/"
    elif tenant.lower() == "earthwatch":
        domain = "https://access.maxar.com/"
    elif tenant.lower() == "evwhs":
        domain = "https://evwhs.digitalglobe.com/"
    else:
        raise Exception('{} is not a valid tenant, please enter a valid tenant'.format(tenant))
    config = ['[ogc]', 'user_tenant = {}'.format(domain.lower()), 'user_connectid = {}'.format(connectid),
              'user_name = {}'.format(username), 'user_password = {}'.format(password)]
    home_dir = os.path.expanduser("~")
    with open(os.path.join(home_dir, ".ogc-config"), "w") as f:
        for line in config:
            f.write(line)
            f.write('\n')


@click.command()
@click.option('--password', '-p', help="New password", type=str, prompt='Enter new password')
def reset_password(password):
    """
    Function updates the password in the configuration file
    """

    home_dir = os.path.expanduser("~")
    with open(os.path.join(home_dir, ".ogc-config"), 'r') as f:
        text = f.readlines()
        text[4] = 'user_password = {}\n'.format(password)
        with open(os.path.join(home_dir, ".ogc-config"), 'w') as f:
            f.writelines(text)


@click.command()
@click.option('--bbox', '-b',
              help='String bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)', type=str)
@click.option('--filter', '-f', help='CQL filter used to refine data of search', type=str)
@click.option('--shapefile', '-s', help='Binary of whether or not to return as shapefile format', type=bool)
@click.option('--featureprofile', '-fp', help='String of the desired stacking profile. Defaults to account Default',
              type=str)
@click.option('--typename', '-t',
              help='String of the typename. Defaults to FinishedFeature. Example input MaxarCatalogMosaicProducts',
              type=str)
def search(bbox=None, filter=None, shapefile=False, featureprofile=None, typename=None):
    """
    Function searches an AOI using the WFS method and returns a list of features and their metadata that intersect with
    the AOI
    """

    check = check_for_config()
    if not typename:
        wfs = check.search(bbox, filter, shapefile, featureprofile=featureprofile)
    else:
        wfs = check.search(bbox, filter, shapefile, featureprofile=featureprofile, typename=typename)
    click.echo(wfs)


@click.command()
@click.option('--bbox', '-b',
              help='String bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)', type=str)
@click.option('--height', '-h', help='Integer value representing the vertical number of pixels to return', type=int)
@click.option('--width', '-w', help='Integer value representing the horizontal number of pixels to return', type=int)
@click.option('--img_format', '-img', help='String of the format of the response image either jpeg, png or geotiff',
              type=str)
@click.option('--identifier', '-id', help='String of the feature id', type=str)
@click.option('--gridoffsets', '-g', help='Sting of the pixel size to be returned in X and Y dimensions', type=str)
@click.option('--zoom_level', '-z', help='Integer value of the zoom level. Used for WMTS', type=int)
@click.option('--download', '-d', help='Boolean of user option to download file locally', type=bool)
def download(bbox=None, height=None, width=None, img_format=None, identifier=None, gridoffsets=None, zoom_level=None,
             download=True):
    """
    Function downloads an image or a list of image calls and returns the file location of the download. NOTE: Structure
    of call should be structured with one of the following:
    1) bbox, zoom_level, img_format
    2) bbox, identifier, gridoffsets, img_format
    3) bbox, img_format, height, width
    """

    check = check_for_config()
    if zoom_level:
        if not bbox:
            raise Exception('zoom_level must have a bbox')
        else:
            wmts = cli_interface.download_image(zoom_level=zoom_level, bbox=bbox, img_format=img_format)
        click.echo(wmts)
    elif identifier:
        if not gridoffsets or not bbox:
            raise Exception('Identifiers must have gridoffset and bbox')
        else:
            wcs = cli_interface.download_image(bbox=bbox, identifier=identifier, gridoffsets=gridoffsets,
                                               img_format=img_format)
        click.echo(wcs)
    else:
        if not bbox or not img_format or not width or not height:
            raise Exception('height/width must have a bbox and an img_format')
        else:
            wms = cli_interface.download_image(bbox=bbox, img_format=img_format,
                                               height=height, width=width, display=False)
        click.echo(wms)


@click.command()
@click.option('--bbox', '-b',
              help='String bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)', type=str)
@click.option('--featureid', '-id', help='String of the feature id', type=str)
@click.option('--band_combination', '-band', help='String of the desired band combination of 1-4 items ex: "R, G, B"',
              type=str)
@click.option('--height', '-h', help='Integer value representing the vertical number of pixels to return', type=int)
@click.option('--width', '-w', help='Integer value representing the horizontal number of pixels to return', type=int)
@click.option('--img_format', '-img', help='String of the format of the response image either jpeg, png or geotiff',
              type=str)
def band_manipulation(bbox=None, featureid=None, band_combination=None, height=256, width=256, img_format='jpeg'):
    """
    Function manipulates the bands of a SWIR 8-band or MS1_MS2 image and returns the file location of the downloaded
    image
    """

    check = check_for_config()
    band_combination = sys.argv[6].split(',')
    band_combination = [i.strip(' ') for i in band_combination]
    bands = cli_interface.band_manipulation(bbox=bbox, featureid=featureid, band_combination=band_combination,
                                            height=height, width=width, img_format=img_format)
    click.echo(bands)


@click.command()
@click.option('--bbox', '-b',
              help='String bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)', type=str,
              prompt='Enter bbox in miny,minx,maxy,maxx format')
def calculate_bbox_sqkm(bbox=None):
    """
    Function calculates the area in square kilometers of the bbox
    """

    check = check_for_config()
    sqkm = cli_interface.calculate_sqkm(bbox=bbox)
    click.echo("Square kilometers of bbox is: {}".format(sqkm))



import requests
import Maxar_OGC.process as process



class WMS:
    def __init__(self, session):
        self.base_url = session['base_url'] + "mapservice/wmsaccess"
        self.headers = session['headers']
        self.connect_id = session['connectid']
        self.response = None
        self.version = session['version']
        self.querystring = self._init_querystring()



    def return_image(self, **kwargs):
        """
        Function finds the imagery matching a bbox or feature id
        Kwargs:
            bbox = String bounding box of AOI. Comma delimited set of coordinates. (miny,minx,maxy,maxx)
            filter = CQL filter used to refine data of search.
            height = Integer value representing the vertical number of pixels to return
            width = Integer value representing the horizontal number of pixels to return
            layers = String representing the called upon layer. Defaults to 'DigitalGlobe:Imagery'
            format = String of the format of the response image either jpeg, png or geotiff
            featureprofile = String of the desired stacking profile. Defaults to account Default
        Returns:
            requests response object of desired image
        """

        self.querystring = self._init_querystring()
        keys = list(kwargs.keys())
        if 'bbox' in keys:
            process._validate_bbox(kwargs['bbox'])
            self.querystring.update({'bbox': kwargs['bbox']})
        else:
            raise Exception('Search function must have a BBOX.')
        if 'filter' in keys:
            self.querystring.update({'coverage_cql_filter': kwargs['filter']})
            del (kwargs['filter'])
        for key, value in kwargs.items():
            self.querystring[key] = value
        request = requests.get(self.base_url, headers=self.headers, params=self.querystring)
        self.response = request
        return process._response_handler(self.response)

    def _init_querystring(self):
        querystring = {'connectid': self.connect_id,
                       'service': 'WMS',
                       'request': 'GetMap',
                       'version': '1.3.0',
                       'crs': 'EPSG:4326',
                       'height': '512',
                       'width': '512',
                       'layers': 'DigitalGlobe:Imagery',
                       'format': 'image/jpeg',
                       'SDKversion': '{}'.format(self.version)
                       }
        return querystring
